@@ -52,10 +52,10 @@ class AnalysisObject(AbstractObject):
     
     def preset_parameters(self):
         ROOT.RooStats.SetAllConstant(self.model.global_observables, True)
-        ROOT.RooStats.SetAllConstant(self.model.nuisance_parameters, False)
+        #ROOT.RooStats.SetAllConstant(self.model.nuisance_parameters, False)
         ROOT.RooStats.SetAllConstant(self.model.pois, True)
         self.stdout.info("INFO: Preset POIs as constant parameters.")
-        self.stdout.info("INFO: Preset nuisance parameters as floating parameters.")
+        #self.stdout.info("INFO: Preset nuisance parameters as floating parameters.")
         self.stdout.info("INFO: Preset global observables as constant parameters.")
     
     @property
