@@ -17,7 +17,7 @@ import ROOT
 import quickstats
 from quickstats import semistaticmethod
 from quickstats.components import AbstractObject
-from quickstats.maths.numerics import is_integer, pretty_value
+from quickstats.maths.numerics import is_integer, pretty_value, get_bins_given_edges, array_issubset
 from quickstats.utils.root_utils import load_macro
 from quickstats.utils.common_utils import str_list_filter
 from quickstats.utils.roofit_utils import (get_variable_attributes, variable_collection_to_dataframe,
@@ -482,8 +482,13 @@ class ExtendedModel(AbstractObject):
                                                          strip_disconnected=strip_disconnected)
         return unfolded_constraints
     
-    def pair_constraints(self, fmt:str="list", to_str=False, sort:bool=True):
+    def pair_constraints(self, fmt:str="list", to_str=False, sort:bool=True, base_component:bool=False):
         constraint_pdfs = self.unfold_constraints()
+        if base_component:
+            # for glob value matching
+            return rf_ext.pair_constraints_base_component(constraint_pdfs, self.nuisance_parameters,
+                                                          self.global_observables, fmt=fmt,
+                                                          sort=sort, to_str=to_str)
         return rf_ext.pair_constraints(constraint_pdfs, self.nuisance_parameters,
                                        self.global_observables, fmt=fmt,
                                        sort=sort, to_str=to_str)
@@ -783,7 +788,6 @@ class ExtendedModel(AbstractObject):
     
     @staticmethod
     def get_pdf_map(pdf_dict:Dict):
-        ExtendedModel.load_RooFitObjects()
         pdfmap = ROOT.std.map('string, RooAbsPdf*')()
         pdfmap.keepalive = list()
         for c, d in pdf_dict.items():
@@ -838,18 +842,16 @@ class ExtendedModel(AbstractObject):
         return asimov_data
     
     def match_globs(self):
-        pdf_list, nuis_list, globs_list = self.pair_constraints(base_component=True)
-        for np, glob in zip(nuis_list, globs_list):
-            glob.setVal(np.getVal())
+        _, components, globs = self.pair_constraints(fmt="argset", base_component=True, sort=True)
+        for component, glob in zip(components, globs):
+            glob.setVal(component.getVal())
             self.stdout.debug("set glob {} to val {}".format(glob.GetName(), glob.getVal()))
         """
         for nuis, globs, pdf in zip(nuis_list, globs_list, pdf_list):
             pdf_class = pdf.ClassName()
             if pdf_class in ["RooGaussian", "RooBifurGauss"]:
                 globs.setVal(nuis.getVal())
-            elif pdf_class == "RooPoisson":
-                from pdb import set_trace
-                set_trace()                
+            elif pdf_class == "RooPoisson":            
                 globs.setVal(nuis.getVal()*globs.getVal())
             else:
                 raise RuntimeError(f"constraint term \"{pdf.GetName()}\" has unsupported type \"{pdf_class}\"")
@@ -1029,7 +1031,7 @@ class ExtendedModel(AbstractObject):
         else:
             asimov_data_map = {}
             channel_cat = sim_pdf.indexCat()
-            n_cat = len(channel_cat)
+            n_cat = channel_cat.size()
             for i in range(n_cat):
                 channel_cat.setIndex(i)
                 label = channel_cat.getLabel()
@@ -1112,7 +1114,8 @@ class ExtendedModel(AbstractObject):
         return toys
     
     def save(self, fname:str, recreate:bool=True):
-        self.workspace.writeToFile(fname, recreate)
+        new_ws = ROOT.RooWorkspace(self.workspace)
+        new_ws.writeToFile(fname, recreate)
     
     @semistaticmethod
     def load_ws(self, fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
@@ -1155,7 +1158,7 @@ class ExtendedModel(AbstractObject):
             raise ValueError("input pdf is not a simultaneous pdf")            
         category_map = {}
         index_cat = pdf.indexCat()
-        n_cat = len(index_cat)
+        n_cat = index_cat.size()
         for i in range(n_cat):
             index_cat.setIndex(i)
             label = index_cat.getLabel()
@@ -1228,12 +1231,26 @@ class ExtendedModel(AbstractObject):
                 elif len(x) == default_bins:
                     binned = True
                 else:
-                    raise RuntimeError(f"detected dataset with invalid binning")
+                    # check for blinded histogram
+                    nbins     = binnings[cat]['bins']
+                    bin_range = binnings[cat]['bin_range']
+                    correct_bins = get_bins_given_edges(bin_range[0], bin_range[1], nbins)
+                    issubset = array_issubset(correct_bins, x)
+                    if issubset:
+                        binned = True
+                        missing_bins   = np.setdiff1d(correct_bins, x)
+                        # fill missing bins with zero values
+                        missing_values = np.zeros(missing_bins.shape[0])
+                        x = np.concatenate([x, missing_bins])
+                        y = np.concatenate([y, missing_values])
+                        idx = np.argsort(x)
+                        x = x[idx]
+                        y = y[idx]
+                    else:
+                        raise RuntimeError(f"detected dataset with invalid binning")
             bins = binnings[cat]['bins']
             if binned:
                 if  (bins != default_bins):
-                    from pdb import set_trace
-                    set_trace()
                     raise RuntimeError(f"cannot change binning of binned dataset ({dataset.GetName()}, cat = {cat}): "
                                        f"original binning = {default_bins}, requested binning = {bins}")
                 distributions[cat]['x'] = x
@@ -1263,7 +1280,7 @@ class ExtendedModel(AbstractObject):
             raise ValueError("input pdf is not a simultaneous pdf")
         distributions = {}
         channel_cat = pdf.indexCat()
-        n_cat = len(channel_cat)
+        n_cat = channel_cat.size()
         binnings = self._get_new_binnings(n_bins)
         for i in range(n_cat):
             channel_cat.setIndex(i)
@@ -1435,6 +1452,14 @@ class ExtendedModel(AbstractObject):
             variables.remove(nuisance_parameters)
             variables.remove(global_observables)
             variables.remove(observables)
+        elif _variable_type == WSVariables.NON_CONSTRAINT_VARIABLES:
+            variables           = self.get_variables("variables")
+            nuisance_parameters = self.get_variables("nuisance_parameters")
+            global_observables  = self.get_variables("global_observables")
+            observables         = self.get_variables("observables")
+            variables.remove(nuisance_parameters)
+            variables.remove(global_observables)
+            variables.remove(observables)
         elif _variable_type == WSVariables.CORE:
             variables = ROOT.RooArgSet()
             variables.add(self.nuisance_parameters)
@@ -1478,6 +1503,14 @@ class ExtendedModel(AbstractObject):
                 raise RuntimeError(f"cannot rename dataset from \"{old_name}\" to \"{new_name}\": "
                                    f"the dataset \"{new_name}\" already exists in the workspace")
             dataset.SetName(new_name)
+    
+    @staticmethod
+    def _format_category_summary(category_name:str, category_map:Dict):
+        observable = category_map['observable']
+        bin_range = category_map['bin_range']
+        bins = category_map['bins']
+        summary_str = f"{category_name} (observable = {observable}, range = [{bin_range[0]}, {bin_range[1]}], bins = {bins})"
+        return summary_str
         
     def print_summary(self, items:Optional[List]=None, suppress_print:bool=False, detailed:bool=True,
                       include_patterns:Optional[List]=None, exclude_patterns:Optional[List]=None,
@@ -1496,10 +1529,13 @@ class ExtendedModel(AbstractObject):
             summary_str += f"Datasets ({len(datasets)}):\n"
             summary_str += "".join([f"\t{obs.GetName()}\n" for obs in datasets])
         # snapshots
-        if ('snapshot' in items) and (quickstats.root_version >= (6, 26, 0)):
-            snapshots = self.workspace.getSnapshots()
-            summary_str += f"Snapshots ({len(snapshots)}):\n"
-            summary_str += "".join([f"\t{snap.GetName()}\n" for snap in snapshots])
+        if ('snapshot' in items):
+            if (quickstats.root_version >= (6, 26, 0)):
+                snapshots = self.workspace.getSnapshots()
+                summary_str += f"Snapshots ({len(snapshots)}):\n"
+                summary_str += "".join([f"\t{snap.GetName()}\n" for snap in snapshots])
+            else:
+                self.stdout.warning("WARNING: Snapshot listing is only available after ROOT 6.26/00")
         # categories
         if 'category' in items:
             if isinstance(self.pdf, ROOT.RooSimultaneous):
@@ -1507,10 +1543,7 @@ class ExtendedModel(AbstractObject):
                 n_cat = len(category_map)
                 summary_str += f"Categories ({n_cat}):\n"
                 for category in category_map:
-                    observable = category_map[category]['observable']
-                    bin_range = category_map[category]['bin_range']
-                    bins = category_map[category]['bins']
-                    summary_str += f"\t{category} (observable = {observable}, range = [{bin_range[0]}, {bin_range[1]}], bins = {bins})\n"
+                    summary_str += "\t" + self._format_category_summary(category, category_map[category]) + "\n"
         # pois, NPs
         param_strs     = []
         param_sets     = []
