@@ -11,7 +11,6 @@ import fnmatch
 from itertools import repeat
 from typing import List, Optional
 
-import uproot
 import numpy as np
 
 from quickstats.components import ExtendedModel, ExtendedMinimizer
@@ -279,13 +278,19 @@ class NuisanceParameterPull(object):
                   eigen:bool=False, strategy:int=0, print_level:int=1, fix_cache:bool=True, fix_multi:bool=True,
                   offset:bool=True, optimize:int=2, parameter:str="", max_calls:int=-1, max_iters:int=-1,
                   batch_mode:bool=False, int_bin_precision:float=-1.,parallel:int=0, cache:bool=True,
-                  exclude:str='', **kwargs):
+                  exclude:str='', constrained_only:bool=True, **kwargs):
         
         # configure default minimizer options
         ExtendedMinimizer._configure_default_minimizer_options(minimizer_type, minimizer_algo,
             strategy, debug_mode=(log_level=="DEBUG"))
         
-        nuis_list = ExtendedModel.get_nuisance_parameter_names(input_file, workspace, model_config)
+        model = ExtendedModel(input_file, workspace, model_config, data_name=data, verbosity="WARNING")
+        # include constrained NPs only
+        if constrained_only:
+            nuis_list = [np.GetName() for np in model.get_variables("constrained_nuisance_parameters")]
+        # include all NPs
+        else:
+            nuis_list = [np.GetName() for np in model.get_variables("nuisance_parameters")]
         include_patterns = parameter.split(',')
         if parameter:
             nuis_to_include = []
@@ -322,6 +327,7 @@ class NuisanceParameterPull(object):
         
     @staticmethod
     def parse_root_result(fname):
+        import uproot
         with uproot.open(fname) as file:
             result = root_utils.uproot_to_dict(file)
         return result
\ No newline at end of file
