@@ -12,6 +12,7 @@ import numpy as np
 
 import ROOT
 
+import quickstats
 from quickstats import semistaticmethod
 from quickstats.components import AbstractObject, ExtendedModel
 from quickstats.utils.io import Verbosity
@@ -44,7 +45,8 @@ class XMLWSModifier(XMLWSBase):
             'define': [],
             'redefine': [],
             'asimov': [],
-            'constraint': []
+            'constraint': [],
+            'add_product_terms': {},            
         }
         self.actions['rename'] = {
             'workspace'   : {},
@@ -64,6 +66,9 @@ class XMLWSModifier(XMLWSBase):
             self.parse_config_dict(source)
         else:
             raise ValueError(f"invalid input format: {source}")
+        if len(self.actions['add_product_terms']) > 0:
+            if (quickstats.root_version < (6, 26, 0)):
+                raise RuntimeError("\"add product terms\" is only supported after ROOT 6.26")
             
     def parse_config_dict(self, source:str):
         _actions = source.pop("actions", {})
@@ -92,7 +97,8 @@ class XMLWSModifier(XMLWSBase):
             "define"     : _actions.pop("define", []),
             "redefine"   : _actions.pop("redefine", []),
             "asimov"     : _actions.pop("asimov", []),
-            "constraint" : []
+            "constraint" : [],
+            "add_product_terms": _actions.pop("add_product_terms", {})
         }
         
         # handle constraint definition
@@ -123,6 +129,7 @@ class XMLWSModifier(XMLWSBase):
             unknown_attributes = list(_rename)
             self.stdout.warning(f"WARNING: Unknown object type in rename: {', '.join(unknown_attributes)}. "
                                 "Ignoring...")
+        
         self.config = config
         self.actions = actions
     
@@ -186,6 +193,13 @@ class XMLWSModifier(XMLWSBase):
             subnodes = node['child']
             for subnode in subnodes:
                 self.parse_rename_node(subnode)
+        elif tag == "AddProductTerm":
+            name = self._get_node_attrib(node, "Name", required=True)
+            extra_terms = self._get_node_attrib(node, "Terms", required=True, dtype="str_list")
+            if name in self.actions['add_product_terms']:
+                raise ValueError("AddProductTerm action defined multiple times for the variable "
+                                 f"\"{name}\"")
+            self.actions['add_product_terms'][name] = extra_terms
         else:
             raise RuntimeError(f"unknown item `{tag}`")
     
@@ -287,6 +301,7 @@ class XMLWSModifier(XMLWSBase):
         title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
         self.stdout.info(title_str)
         rename_map = self.get_rename_map(ws_orig, ws_tmp)
+        actual_rename_map = self.get_actual_rename_map(rename_map, ws_orig, ws_tmp)
         self.rename_variables(mc_orig, ws_tmp, rename_map)
         if not rename_map:
             self.stdout.info("INFO: No variables are renamed.")
@@ -301,9 +316,12 @@ class XMLWSModifier(XMLWSBase):
             self.import_object(ws_final, sim_pdf)
             self.import_datasets(ws_orig, ws_final)
         else:
-            ws_final = ws_tmp
+            ws_final = ws_tmp        
+            
+        # post processing pdfs, functions and variables
+        self.post_process_workspace(ws_final)
 
-        mc_final = self.create_model_config(mc_orig, ws_final, rename_map)
+        mc_final = self.create_model_config(mc_orig, ws_final, actual_rename_map)
         
         self.rename_objects(ws_final, mc_final)
         
@@ -323,6 +341,22 @@ class XMLWSModifier(XMLWSBase):
         ws_final.writeToFile(outname, True)
         self.stdout.info(f"INFO: Saved output workspace as \"{outname}\".")
     
+    def post_process_workspace(self, ws:ROOT.RooWorkspace):
+        self.add_product_terms(ws)
+    
+    def add_product_terms(self, ws:ROOT.RooWorkspace):
+        for target_name, extra_terms in self.actions["add_product_terms"].items():
+            prod_var = self.get_workspace_arg(ws, target_name, strict=True)
+            if not isinstance(prod_var, ROOT.RooProduct):
+                class_name = prod_var.ClassName()
+                raise RuntimeError(f"failed to add product term to the variable \"{target_name}\": "
+                                   f"expect a RooProduct instance but {class_name} received")
+            for extra_term in extra_terms:
+                member_var = self.get_workspace_arg(ws, extra_term, strict=True)
+                prod_var.addTerm(member_var)
+            self.stdout.info(f"INFO: The following product terms are added to the variable {target_name}: "
+                             f"{', '.join(extra_terms)}")
+    
     def setup_parameters(self, ws:ROOT.RooWorkspace):
         model = ExtendedModel(ws, data_name=None, verbosity="WARNING")
         model.stdout.verbosity = "INFO"
@@ -662,9 +696,19 @@ class XMLWSModifier(XMLWSBase):
             raise RuntimeError(f"pdf {pdf_names} not found in the workspace loaded from {ext_rfile}")
         self.import_object(ws, pdf)
     
+    def get_actual_rename_map(self, rename_map:Dict, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace):
+        actual_rename_map = {}
+        for old_name, new_name in rename_map.items():
+            # if the object with the new name isalready defined, it's not a renaming
+            if (old_ws.arg(new_name)) or (new_ws.arg(new_name)):
+                continue
+            actual_rename_map[old_name] = new_name
+        return actual_rename_map
+    
     def get_rename_map(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace):
         rename_map = {}
-        regex = re.compile(r"\(([\w=,]+)\)")
+        #regex = re.compile(r"\(([\w=,]+)\)")
+        regex = re.compile(r"\((.+)\)")
         strict = self.config['strict']
         # rename definitions from the "Map" node
         for expr in self.actions['map']:
