@@ -71,8 +71,9 @@ class ParamParser:
 
     def __init__(self, format_str:Optional[str]=None,
                  param_str:Optional[str]=None,
-                 file_ext:Optional[str]=None):
-        
+                 file_ext:Optional[str]=None,
+                 allow_none:bool=True):
+        self.allow_none = allow_none
         self.setup(format_str, param_str, file_ext)
     
     def setup(self, format_str:Optional[str]=None,
@@ -142,10 +143,11 @@ class ParamParser:
         param_expr_list = [s for s in param_expr_list if s]
         for expr in param_expr_list:
             tokens = expr.split('=')
-            if len(tokens) != 2:
+            if len(tokens) not in [1, 2]:
                 raise ValueError('invalid expression for parameterisation')
             attribute = tokens[0]
-            attributes.append(attribute)
+            if attribute not in attributes:
+                attributes.append(attribute)
         return attributes
     
     @semistaticmethod
@@ -164,6 +166,14 @@ class ParamParser:
         param_expr_list = [s for s in param_expr_list if s]
         for expr in param_expr_list:
             tokens = expr.split('=')
+            # floating poi
+            if len(tokens) == 1:
+                param_name = tokens[0]
+                if param_name in param_values:
+                    raise RuntimeError(f"profiled parameter {param_name} appeared more than once in the parameter "
+                                       f"expression: {param_str}")
+                param_values[param_name] = [None]
+                continue
             if len(tokens) != 2:
                 raise ValueError('invalid expression for parameterisation')
             param_name = tokens[0]
@@ -180,7 +190,12 @@ class ParamParser:
                 values = np.arange(poi_min, poi_max + poi_step, poi_step)
             else:
                 raise ValueError('invalid expression for parameterisation')
-            param_values[param_name] = values
+            if param_name not in param_values:
+                param_values[param_name] = np.array([])
+            if None in param_values[param_name]:
+                raise RuntimeError(f"the parameter {param_name} is being profiled and non-profiled at the same time "
+                                   f"in the parameter expression: {param_str}")
+            param_values[param_name] = np.concatenate([param_values[param_name], values])
         param_names = list(param_values.keys())
         combinations = [param_values[param_name] for param_name in param_names]
         combinations = itertools.product(*combinations)
@@ -192,7 +207,14 @@ class ParamParser:
     
     @staticmethod
     def val_encode_parameters(parameters:Dict):
-        return ",".join([f"{param}={round(value, 8)}" for param, value in parameters.items()])
+        tokens = []
+        for param, value in parameters.items():
+            if value is None:
+                token = f"{param}"
+            else:
+                token = f"{param}={round(value, 8)}"
+            tokens.append(token)
+        return ",".join(tokens)
     
     @staticmethod
     def str_encode_parameters(parameters:Dict):
@@ -286,13 +308,21 @@ class ParamParser:
                 keep &= not (self.select_param_point(parameters, exclude_conditions, default=False))
             if keep:
                 selected_param_points.append(param_point)
-        return selected_param_points    
+        return selected_param_points
+    
+    def sanity_check(self, internal_param_points:Dict, external_param_points:Dict):
+        if not self.allow_none:
+            for int_point in internal_param_points:
+                for param_name, param_value in int_point.items():
+                    if param_value is None:
+                        raise RuntimeError(f"profiled parameter is not allowed ({param_name})")
     
     def get_param_points(self, dirname:str="",
                          filter_expr:Optional[str]=None,
                          exclude_expr:Optional[str]=None):
         external_param_points = self.get_external_param_points(dirname, filter_expr, exclude_expr)     
         internal_param_points = self.get_internal_param_points(filter_expr, exclude_expr)
+        self.sanity_check(internal_param_points, external_param_points)
         if len(internal_param_points) == 0:
             internal_param_points = [{}]
         param_points = []
