@@ -52,16 +52,21 @@ def release_tfiles():
     for f in ROOT.gROOT.GetListOfFiles():
         f.Close()
 
-def compile_macro(name:str):
+def compile_macro(name:str, opt:str="kfOc"):
     macros_dir = os.path.join(quickstats.macro_path, 'macros', name)
     macros_path = os.path.join(macros_dir, name + '.cxx')
     print('INFO: Compiling macro "{}"...'.format(name))
     if not os.path.exists(macros_path):
         print('ERROR: Cannot locate macro files from {}. ROOT macros will not be compiled.'.format(macros_dir))
         return -1
-    return ROOT.gROOT.LoadMacro('{}++'.format(macros_path))
+    status = ROOT.gSystem.CompileMacro(macros_path, opt)
+    #status = ROOT.gROOT.LoadMacro('{}++'.format(macros_path))
+    return status
 
 def load_macro(name:str):
+    import cppyy
+    if name in dir(cppyy.gbl):
+        return None
     auto_load = ROOT.gInterpreter.AutoLoad(name)
     # already loaded by quickstats / third-party
     if auto_load == 1:
@@ -72,12 +77,13 @@ def load_macro(name:str):
     macros_dir = os.path.join(quickstats.macro_path, 'macros', name)
     macros_path = os.path.join(macros_dir, name + '_cxx')
     if not os.path.exists(macros_path + '.so'):
-        return compile_macro(name)
-    result = ROOT.gSystem.Load(macros_path)
-    if (result != 0) and (result != 1):
+        status_compile = compile_macro(name)
+    status_load = ROOT.gSystem.Load(macros_path)
+    if (status_load != 0) and (status_load != 1):
         raise RuntimeError(f"Shared library for the macro {name} is incompatible with the current pyROOT version. "
-                           "Please recompile by typing \"quickstats compile\".")
-    return result
+                           "Please recompile by typing \"quickstats compile\". "
+                           "(use quickstats.compile_macros() instead inside a jupyter kernel)")
+    return status_load
 
 def get_macro_dir(name:str):
     return os.path.join(quickstats.macro_path, 'macros', name)
